# DSF documentation

DSF is a framework that provides an API to build Dock Screens in pure
[tlisp](#tlisp). It also comes with some common utility functions that can be
used in other mods and defines some common useful screens.

## Limitations

DSF is a very powerful framework, but it has its limitations. Not every
conceivable Dock Screen can be built using DSF without having very good
knowledge of tlisp and how DSF works. The set of standard screens that DSF
provides have no fancy background images for example, so if you want to add that
kind of flavour and use DSF then you would have to make some additions yourself.
See Adding a new screen type for an example. A complex Canvas screen like the
neurohack screen can be hard to build as well. DSF does not come ready to build
such a screen, but, with some work, can be extended to do it.

## Examples

To get you started, here are some simple examples of how to build a screen with DSF

### Simple Example

```lisp
(dsf/new-screen 'example "Example Screen")
(dsf/set-description "This is the initial description")
(dsf/add-action 'update '("Update Description" 'U) (lambda nil
    (block nil
        (dsf/clear-description)
        (dsf/set-description "This is the updated description")
    )
))
```

That is how simple it is to define a basic Dock Screen. Under the hood there is
a lot of stuff going on in DSF, but the basic concept is rather simple. Let us
walk through the above example line by line and have a look at how it works.

#### Creating the screen

With a call to dsf/new-screen we start defining a new screen. We supply a unique
Screen ID that we will use to refer to this screen from now on. We also supply
the text for the title of the screen.  One of the first things that happen upon
creating a new screen is setting it as the active screen.  Almost all API
functions in DSF work only on the active screen. That means we don’t have to
supply the screen id when we use the API functions.

We could also have supplied the type of the screen we wanted to use, but since
we are fine with the default we leave it out. DSF comes with four builtin Screen
Types (Standard, CustomPicker, ItemPicker and Canvas). The default type is
Standard.

##### On Visibility
> It is important to realise that at this point, the screen we are defining is
> not actually being shown on the screen. It is just being defined for later
> use. More on that in the Screen Visibility section.

#### Setting the description

We set an initial description using dsf/set-description. Since we have no fancy
needs we just pass in a string. This string will be used raw when the screen
description is finally set. Notice how we don’t need a screen id. DSF lets us
work easily on the current active screen.

#### Adding the action

We add an action to the screen by calling dsf/add-action. We supply it with
three arguments: an action id, a handle and an expression.

The Action ID is a unique name which serves to identify this action. It need
only be unique for this single screen though. That means you can define multiple
screens with a 'select action, for example.

The handle sets the Action Handle of the action. Typically it will be a tuple
with the actions label as the first item and the key as the second.

The expression we use here is a lambda that contains the script we would like to
run when the action is invoked. As you can see, when that happens, we first
clear the current description and then set a new one. Since this is happening
inside an action, the pane is going to be updated at the end, with the result
that a new description will be shown.

This is just a small example of what an action can do. You can actually do just
about anything inside an action, even define entire new screens.

#### Showing the screen

Like stated in the note above, the screen is not actually visible at this point.
It is just stored as a set of data that DSF can use to show a screen.

When the time comes that you want to show your screen, all it takes is simply a
call to dsf/show-screen. If you want to show a specific screen then you can pass
in the screen id, but often you will just want to show the active screen. So,
all it would take to show the screen we defined in the example above, is to
follow up with a simple:

`(dsf/show-screen)`

That would pop up a Dock Screen with the contents of our 'example screen.

You might notice how we don’t define an Exit action or a Back action, yet both
appear. DSF automatically adds those two actions when applicable, unless you
specifically tell it not to.  For more information see [Default Actions]

That concludes our simple example. Next we will build one of the default
Transcendence screens in DSF, so we can compare them side by side.

#### Default Actions

Info about default actions and how to disable/configure them

### tlisp

This section gives a brief introduction to tlisp and links to some further
references

The game [Transcendence](http://neurohack.com/transcendence/) is &copy; George
Moromisato 1999 - 2011
